#!/bin/bash
# Drop traffic from unwanted IPv4 and IPv6-adresses by inserting them into the firewall
# or removing the rule again. Non-permanently or permanently.
# If non-permanently the rules may be reapplied after boot by executing the commands as logged into $LOGFILE
#
# parameter: [A|R] {net/host|.} [port[-endport]]
# A add / R remove rule into/from the zone drop (non-permanently)
# Addresses may also be IPv6 addresses
#
# 2022-08-29 J. Thomsen bugs@jth.net

# Drop all traffic from 1.2.3.4/24
# ip_drop.sh A 1.2.3.4/24
# Drop all traffic to port 1234
# ip_drop.sh A . 1234
# Drop all traffic from 1.2.3.4 to port 1234
# ip_drop.sh A 1.2.3.4 1234
# Drop all traffic from  2abc:2:3::4 to the ports 1234-1244
# ip_drop.sh A 2abc:2:3::4 1234-1244

# Useful commands:
# Reload the permanent configuration and forget non-permanent changes
# firewall-cmd --reload
# Remove all drop rules
# firewall-cmd --permanent --load-zone-defaults=drop
# Make the changes made to the running configuration permanent
# firewall-cmd --runtime-to-permanent

PERM=--permanent
PERM=

# Change to the preferred full path
LOGFILE=/root/droplist.txt
# The logfile "$LOGFILE" may be executed at boot by sh /root/droplist.txt
# (see onboot.service and onboot.sh)
# or the PERM parameter may be set to --permanent if preferred

function fwinsrule() {
# A/D {<1.2.3.4>|""} [<port>]
FN=$1
IP=$2
PORT=$3
if [[ "$IP" =~ ":" ]]; then
	FAM=ipv6
else
	FAM=ipv4
fi

RC=0
if [[ "$FN" == "D" ]]; then
	FN=remove
else
	FN=add
fi

if [[ "${IP}x" != ".x" && "${PORT}x" != "x" ]]; then
	firewall-cmd $PERM --zone=drop --${FN}-rich-rule='rule family="$FAM" source address="'$IP'" port port="'$PORT'" protocol="tcp" reject'
	RC=$?
	echo firewall-cmd $PERM --zone=drop --$FN-rich-rule='rule family="$FAM" source address="'$IP'" port port="'$PORT'" protocol="tcp" reject'  >>"$LOGFILE"
	firewall-cmd $PERM --zone=drop --${FN}-rich-rule='rule family="$FAM" source address="'$IP'" port port="'$PORT'" protocol="udp" reject'
	if [[ "$RC" == "0" ]]; then RC=$?; fi
	echo firewall-cmd $PERM --zone=drop --$FN-rich-rule='rule family="$FAM" source address="'$IP'" port port="'$PORT'" protocol="udp" reject'  >>"$LOGFILE"
else
if [[ "${IP}x" == ".x" && "${PORT}x" != "x" ]]; then
	firewall-cmd $PERM --zone=drop --$FN-port=$PORT/tcp
	RC=$?
	echo firewall-cmd $PERM --zone=drop --$FN-port=$PORT/tcp  >>"$LOGFILE"
	firewall-cmd $PERM --zone=drop --$FN-port=$PORT/udp
	if [[ "$RC" == "0" ]]; then RC=$?; fi
	echo firewall-cmd $PERM --zone=drop --$FN-port=$PORT/udp  >>"$LOGFILE"
else
if [[ "${IP}x" != ".x" && "${PORT}x" == "x" ]]; then
	firewall-cmd $PERM --zone=drop --add-source=$IP
	RC=$?
	echo firewall-cmd $PERM --zone=drop --$FN-source=$IP >>"$LOGFILE"
else
	echo "Script error: not found $FN $IP $PORT"
	RC=3
fi
fi
fi
}

ACTION=$1
IP=$2
PORT=$3
RC=4

# a bit clumsy for historical reasons

if [[ "${ACTION}x" == "Rx" ]] ; then
if [[ "${PORT}x" != "x" ]] ; then
if [[ "${IP}x" != ".x" ]] ; then
RC=0
else
RC=0
fi
else
RC=0
fi

else
if [[ "${ACTION}x" == "Ax" ]] ; then
if [[ "${PORT}x" != "x" ]] ; then
if [[ "${IP}x" != ".x" ]] ; then
RC=0
else
RC=0
fi
else
RC=0
fi
else
echo "$0 [A|R] {net/host|.} [port[-endport]]"
exit $RC
fi
fi

if [ $RC = 0 ]; then
	fwinsrule $ACTION $IP $PORT
	if [ $RC != 0 ]; then
		echo "****** Verify the error: $RC"
	fi
fi
exit $RC

